from .download_repo import download_repo
from .load_config import load_config
from .get_dd import get_dd
from .get_id_or_url import get_id_with_url, get_url_with_id
from .child_base_data import TableData, LabelData, DocData, ImageData, AudioData, VideoData, GraphData



# -*- coding:utf-8 -*-

from .base_data import BaseData

"""
按照下面的继承关系实现各个数据类
- BaseData
    - RelationalData
        - TableData
        - LabelData
    - NonRelationalData
        - DocData
        - PhotoData
        - VideoData
        - AudioData
    - GraphData
"""


class RelationalData(BaseData):
    """
    关系型数据类
    """

    def __init__(self, digit_id: str = "defeat"):
        super().__init__(digit_id)


class TableData(RelationalData):
    """
    表格型数据类,集成继承关系型数据类
    """

    def __init__(self, digit_id: str = "defeat"):
        super().__init__(digit_id)


class LabelData(RelationalData):
    """
    标签型数据类，继承关系型数据类s
    """

    def __init__(self, digit_id: str = "defeat"):
        super().__init__(digit_id)


class NonRelationalData(BaseData):
    """
    非关系型数据类
    """

    def __init__(self, digit_id: str = "defeat"):
        super().__init__(digit_id)


class DocData(NonRelationalData):
    """
    文本型数据类
    """

    def __init__(self, digit_id: str = "defeat"):
        super().__init__(digit_id)


class ImageData(NonRelationalData):
    """
    图片型数据类
    """

    def __init__(self, digit_id: str = "defeat"):
        super().__init__(digit_id)


class VideoData(NonRelationalData):
    """
    视频型数据类
    """

    def __init__(self, digit_id: str = "defeat"):
        super().__init__(digit_id)


class AudioData(NonRelationalData):
    """
    音频型数据类
    """

    def __init__(self, digit_id: str = "defeat"):
        super().__init__(digit_id)


class GraphData(BaseData):
    """
    图数据类
    """

    def __init__(self, digit_id: str = "defeat"):
        super().__init__(digit_id)



import json

from .load_setting import load_setting

"""
加载配置文件
"""


def load_config(id):
    """
    加载数据对应的config.json配置文件
    :param: id 数据id
    :return: dict 返回config.json文件的内容
    """
    SETTING = load_setting()
    with open(f'{SETTING["base_path"]}repo_{id}/config.json', 'r', encoding='UTF-8') as f:
        return json.load(f)



"""
base_data.py
其中包含BaseData类，用于存储数据
"""
import json
import os


class BaseData(object):
    def __init__(self, digit_id: str = "default"):
        if digit_id == "default":
            self.digit_id = self.auto_digit_id()
        else:
            self.digit_id = digit_id
        self.config = self.load_config()
        self.data = self.load_data()

    def auto_digit_id(self):
        """
        自动找到 digit_id
        """
        # 1. 扫描download_repo文件夹下的所有文件夹
        base_path = "./download_repo"
        if not os.path.exists(base_path):
            return "download_repo文件夹不存在"
        resource = os.listdir(base_path)
        # 2. 查找其文件夹名字，根据repo_digit_id规则拆分出digit_id
        digit_ids = []
        for r in resource:
            if r.startswith("repo_"):
                digit_ids.append(r.split("_")[1])
        # 3. 如果有多个digit_id，返回“download_repo下有多个资源，请指定资源digit_id”
        if len(digit_ids) > 1:
            return "download_repo下有多个资源，请指定资源digit_id后重试"
        # 4. 如果没有digit_id，返回“download_repo下没有资源，请先下载资源”
        elif len(digit_ids) == 0:
            return "download_repo下没有资源，请使用 def download_repo() 下载资源"
        # 5. 如果只有一个digit_id，返回该digit_id
        else:
            return digit_ids[0]

    def load_config(self):
        """
        加载配置文件
        """

        config_path = f"./download_repo/repo_{self.digit_id}/config.json"
        if not os.path.exists(config_path):
            return f"配置文件不存在，请检查路径：{config_path}"
        with open(config_path, 'r', encoding='utf-8') as f:
            return json.load(f)

    def data_guide(self):
        """
        数据使用说明,包括数据集说明和代码说明
        默认使用config.json中的description字段
        可自行重构方法
        """
        data_description = self.config['dataset']['description']
        code_description = self.config['code']['description']
        print(f"dataset instruction：\n{data_description}\n，code instruction：\n{code_description}")
        return {"data_description": data_description, "code_description": code_description}

    def data_scale(self):
        """
        数据规模，默认使用config.json中的data_files字段
        可自行重构方法
        """
        records = self.config['dataset']['data_files']
        for record in records:
            print(f"dataset name: {record['file_name']}\t records: {record['records']}\t size: {record['size']}")

    def load_data(self):
        """
        !!! 必须重构方法
        加载数据
        可自行重构方法
        :return : dataset
        """

        pass

    def data_preview(self):
        """
        数据预览
        """
        pass

    def data_statistics(self):
        """
        描述性统计
        """
        pass

    def data_distribution(self):
        """
        数据分布
        """
        pass

    def data_preprocessing(self):
        """
        数据预处理
        """
        pass



from .load_setting import load_setting
import sys
from .load_config import load_config

def get_dd(id:str = 'default'):
    if id == 'default':
        id = load_config()['id']
    SETTING = load_setting()
    base_path = SETTING['base_path'].strip(".").strip("/")
    sec_path = f'repo_{id}'
    sys.path.append(f"{base_path}/{sec_path}")
    dd = __import__("from {base_path}.{sec_path}.code import DigitData")
    return dd








import setuptools


setuptools.setup(
    name="ir_digit",
    version="0.0.23",
    author="FKLiu",
    author_email="fkliu001@outlook.com",
    description="digit package",
    long_description="digit package ……",
    long_description_content_type="text",
    url="https://example.com",
    packages=setuptools.find_packages(),
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)



import os

import git

from .get_id_or_url import get_url_with_id, get_id_with_url
from .load_setting import load_setting

"""
下载git仓库
"""


def download_repo_git(url, id, base_path, sec_path):
    print('Download start...')
    if not os.path.exists(base_path + sec_path):  # 不存在该路径，创建该路径
        os.mkdir(base_path + sec_path)
        # 下载git仓库
        git.Repo.clone_from(url, base_path + sec_path)

    else:
        # 存在该路径则清空该路径下的所有文件
        for file in os.listdir(base_path + sec_path):
            if not file:  # 无旧数据
                # 下载git仓库
                git.Repo.clone_from(url, base_path + sec_path)
            else:
                # 如果有旧数据，则拉取git仓库最新更新
                git.Repo(base_path + sec_path).remotes.origin.pull()
    print('Successfully download/update, the files are as follows:')
    print(os.listdir(base_path + sec_path))

    return id  # 返回id，用于判断后续数据使用路径


def download_repo(id_or_url: str, url_type='git'):
    url = ''
    id = ''











































# 据输入的id或url判断是id还是url，并获取对应的url和id
    if len(id_or_url) == 6:
        id = id_or_url
        url = get_url_with_id(id)
    else:
        url = id_or_url
        id = get_id_with_url(url)
    SETTING = load_setting()
    base_path = SETTING['base_path']
    sec_path = f'repo_{id}/'

    if not os.path.exists(base_path):  # 不存在该base_path路径，创建该路径
        os.mkdir(base_path)

    if url_type == 'git':
        download_repo_git(url, id, base_path, sec_path)
